@@ -21,11 +21,14 @@
 #' @param from,to text dates in format \code{"YYYY-mm-dd"}
 #' @param period candle period \code{tick, 1min, 5min, 10min, 15min, 30min, hour, day, week, month}
 #' @param split.adjusted should data be split adjusted?
-#' @param local should data be loaded from local storage? Only 'tick' period supported for local storage. See 'Details' section
+#' @param local should data be loaded from local storage? See 'Details' section
+#' @param code futures or option code name, e.g. \code{"RIU6"}
+#' @param contract,frequency,day_exp same as in \code{\link{gen_futures_codes}}
 #' @name get_market_data
 #' @details
 #' Use external websites to get desired symbol name for
 #' \href{https://www.finam.ru/profile/moex-akcii/sberbank/export/}{Finam},
+#' \href{https://www.moex.com/en/derivatives/contracts.aspx}{MOEX},
 #' \href{https://www.iqfeed.net/symbolguide/index.cfm?symbolguide=lookup}{IQFeed},
 #' \href{https://finance.yahoo.com/}{Yahoo} and
 #' \href{https://www.google.com/finance}{Google} sources. \cr
@@ -34,7 +37,8 @@
 #' Load time is reduced dramatically. It is a good way to collect market data as
 #' e.g. IQFeed gives only 180 days of tick data if you would need more it will
 #' cost you a lot. See \code{\link{store_market_data}} for details. \cr
-#' See \link{iqfeed} return format specification.
+#' See \link{iqfeed} return format specification. \cr
+#' MOEX data can be retrieved from local storage only in order to minimize load on MOEX data servers. Read \code{\link{store_market_data}} for information on how to store data locally. \cr
 #'
 #' @examples
 #' \donttest{
@@ -49,6 +53,10 @@
 #' get_google_data( 'MSFT', '2015-01-01', '2016-01-01' )
 #' get_yahoo_data( 'MSFT', '2015-01-01', '2016-01-01' )
 #'
+#' get_moex_futures_data( 'RIH9', '2009-01-01', '2009-02-01', 'tick', local = T )
+#' get_moex_options_data( 'RI55000C9', '2009-01-01', '2009-02-01', 'tick', local = T )
+#' get_moex_continuous_futures_data( 'RI', '2016-01-01', '2016-11-01', frequency = 3, day_exp = 15 )
+#'
 #' }
 # split character date into desired parts
 NULL
@@ -72,6 +80,10 @@ NULL
 #' @export
 get_yahoo_data = function( symbol, from, to, split.adjusted = TRUE ) {
 
+  curr_date = format( Sys.Date() )
+  if( from > curr_date ) from = to = curr_date
+  if( to   > curr_date ) to = curr_date
+
   splits = get_yahoo_splits_and_dividends( symbol, from, to )
   # split dates into parts
   from = .extract_date_parts( from )
@@ -120,6 +132,10 @@ get_yahoo_data = function( symbol, from, to, split.adjusted = TRUE ) {
 #' @export
 get_yahoo_splits_and_dividends = function( symbol, from, to = from ) {
 
+  curr_date = format( Sys.Date() )
+  if( from > curr_date ) from = to = curr_date
+  if( to   > curr_date ) to = curr_date
+
   # split dates into parts
   from = .extract_date_parts( from )
   to = .extract_date_parts( to )
@@ -152,6 +168,10 @@ get_yahoo_splits_and_dividends = function( symbol, from, to = from ) {
 #' @export
 get_google_data = function( symbol, from, to = from ){
 
+  curr_date = format( Sys.Date() )
+  if( from > curr_date ) from = to = curr_date
+  if( to   > curr_date ) to = curr_date
+
   from = as.Date( from )
   to = as.Date( to )
 
@@ -180,6 +200,11 @@ get_finam_data = function( symbol, from, to = from, period = 'day', local = FALS
     return( data )
 
   }
+
+  curr_date = format( Sys.Date() )
+  if( from > curr_date ) from = to = curr_date
+  if( to   > curr_date ) to = curr_date
+
   # Finam host address
   host = 'export.finam.ru'
   # referer to successfully download data from Finam server
@@ -316,13 +341,41 @@ get_finam_data = function( symbol, from, to = from, period = 'day', local = FALS
 #' @export
 get_iqfeed_data = function( symbol, from, to = from, period = 'day', local = FALSE ) {
 
+  curr_date = format( Sys.Date() )
+  if( from > curr_date ) from = to = curr_date
+  if( to   > curr_date ) to = curr_date
+
   if( local ){
 
-    if( period != 'tick' ) stop( 'only ticks supported in local storage' )
+    if( period == 'tick' ) data = .get_local_data(  symbol, from, to, source = 'iqfeed', period = 'tick' )
+    if( period != 'tick' ) {
 
-    data = .get_local_data(  symbol, from, to, source = 'iqfeed' )
+      data = .get_local_data(  symbol, from, to, source = 'iqfeed', period = '1min' )
+      if( is.null( data ) ) return( NULL )
 
-    return( data )
+      switch( period,
+              '1min'  = { n =  1; units = 'mins' },
+              '5min'  = { n =  5; units = 'mins' },
+              '10min' = { n = 10; units = 'mins' },
+              '15min' = { n = 15; units = 'mins' },
+              '30min' = { n = 30; units = 'mins' },
+              'hour'  = { n =  1; units = 'hour' },
+              'day'   = { n =  1; units = 'days' }
+      )
+
+      open = high = low = close = volume = NULL
+      data = data[ , list( open = open[1], high = max( high ), low = min( low ), close = close[.N], volume = sum( volume ) ), by = list( time = ceiling_POSIXct( time, n, units ) ) ]
+      if( period == 'day' ) {
+
+        data[, time := as.Date( time ) - 1 ]
+        setnames( data, 'time', 'date' )
+
+
+      }
+
+    }
+
+    return( data[] )
 
   }
 
@@ -340,4 +393,64 @@ get_iqfeed_data = function( symbol, from, to = from, period = 'day', local = FAL
   return( data )
 
 }
+#' @rdname get_market_data
+#' @export
+get_moex_options_data = function( code, from, to = from, period = 'tick', local = TRUE ) {
 
+  .get_moex_data( code = code, from = from, to = to, period = period, local = local, type = 'options' )
+
+}
+#' @rdname get_market_data
+#' @export
+get_moex_futures_data = function( code, from, to = from, period = 'tick', local = TRUE ) {
+
+  .get_moex_data( code = code, from = from, to = to, period = period, local = local, type = 'futures' )
+
+}
+#' @rdname get_market_data
+#' @export
+get_moex_continuous_futures_data = function( contract, from, to, frequency, day_exp ) {
+
+  schedule = gen_futures_codes( contract, from, to, frequency, day_exp, year_last_digit = T )
+
+  trades = schedule[, get_moex_futures_data( code, from, to ), by = contract_id ][]
+  setcolorder( trades, c( 'time', 'price', 'volume', 'id', 'contract_id' )  )
+  code = contract_id = NULL
+  trades[, code := schedule$code[ contract_id ] ][]
+  gc()
+  return( trades )
+
+}
+
+.get_moex_data = function( code, from, to = from, period = 'tick', local = TRUE, type = c( 'options', 'futures' ) ) {
+
+  if( period != 'tick' ) stop( 'only \'tick\' data supported' )
+  if( !local ) stop( 'only \'local = TRUE\' flag supported' )
+  type = match.arg( type )
+
+  dir_data = paste0( .settings$moex_storage, '/', type, '/' )
+
+  files = list.files( dir_data, pattern = '.rds', full.names = T )
+  dates = gsub( '.*/|\\..*', '', files )
+
+  data = vector( length( dates[ from <= dates & dates <= to ] ), mode = 'list' )
+  i = 1
+
+  for( file in files[ from <= dates & dates <= to ] ) {
+
+    code_ = code
+
+    dat_time = price = amount = NULL
+
+    Nosystem = 0
+    data[[ i ]] = readRDS( file )[ code == code_ & Nosystem != 1, list( time = dat_time, price, volume = as.integer( amount ), id = 1:.N ) ]
+
+    i = i + 1
+
+  }
+
+  data = rbindlist( data, use.names = T, fill = T )[]
+  gc()
+  return( data )
+
+}
